@@ -614,7 +614,7 @@ static int _init_system_id(struct cmd_context *cmd)
 	return 1;
 }
 
-static void _init_device_ids_refresh(struct cmd_context *cmd)
+static int _init_device_ids_refresh(struct cmd_context *cmd)
 {
 	const struct dm_config_node *cn;
 	const struct dm_config_value *cv;
@@ -622,14 +622,21 @@ static void _init_device_ids_refresh(struct cmd_context *cmd)
 	int check_hostname = 0;
 	char path[PATH_MAX];
 	char uuid[128] = { 0 };
+	int val;
 
 	cmd->device_ids_check_product_uuid = 0;
 	cmd->device_ids_check_hostname = 0;
 
-	if (!find_config_tree_bool(cmd, devices_device_ids_refresh_CFG, NULL))
-		return;
+	if (!(val = find_config_tree_int(cmd, devices_device_ids_refresh_CFG, NULL)))
+		return 1;
 	if (!(cn = find_config_tree_array(cmd, devices_device_ids_refresh_checks_CFG, NULL)))
-		return;
+		return 1;
+
+	/* 0 disables, 1 enables single refresh, 10-600 enables refresh period */
+	if ((val > 1 && val < 10) || (val > 600)) {
+		log_error("Invalid device_ids_refresh value %u", val);
+		return 0;
+	}
 
 	for (cv = cn->v; cv; cv = cv->next) {
 		if (cv->type != DM_CFG_STRING)
@@ -643,7 +650,7 @@ static void _init_device_ids_refresh(struct cmd_context *cmd)
 	if (check_product_uuid) {
 		const char *sysfs_dir = cmd->device_id_sysfs_dir ?: dm_sysfs_dir();
 		if (dm_snprintf(path, sizeof(path), "%sdevices/virtual/dmi/id/product_uuid", sysfs_dir) < 0)
-			return;
+			return_0;
 		if (get_sysfs_value(path, uuid, sizeof(uuid), 0) && uuid[0])
 			cmd->product_uuid = dm_pool_strdup(cmd->libmem, uuid);;
 		if (cmd->product_uuid)
@@ -652,6 +659,7 @@ static void _init_device_ids_refresh(struct cmd_context *cmd)
 
 	if (check_hostname && cmd->hostname)
 		cmd->device_ids_check_hostname = 1;
+	return 1;
 }
 
 static int _process_config(struct cmd_context *cmd)
@@ -834,7 +842,8 @@ static int _process_config(struct cmd_context *cmd)
 	if (!_init_system_id(cmd))
 		return_0;
 
-	_init_device_ids_refresh(cmd);
+	if (!_init_device_ids_refresh(cmd))
+		return_0;
 
 	init_io_memory_size(find_config_tree_int(cmd, global_io_memory_size_CFG, NULL));
 
@@ -1186,7 +1195,7 @@ static int _init_dev_cache(struct cmd_context *cmd)
 	return 1;
 }
 
-#define MAX_FILTERS 10
+#define MAX_FILTERS 20
 
 static struct dev_filter *_init_filter_chain(struct cmd_context *cmd)
 {
