@@ -10,19 +10,19 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include "tools/tool.h"
-
 #include "daemon-server.h"
 #include "daemon-log.h"
+#include "daemon-stray.h"
 #include "libdaemon/client/daemon-io.h"
 
+#include "lib/misc/util.h"
+
 #include <dlfcn.h>
 #include <errno.h>
+#include <fcntl.h> /* help musl C */
 #include <pthread.h>
-#include <sys/file.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
-#include <sys/time.h>
 #include <sys/resource.h>
 #include <netinet/in.h>
 #include <sys/un.h>
@@ -48,9 +48,9 @@ static int _pthread_create(pthread_t *t, void *(*fun)(void *), void *arg, int st
 static volatile sig_atomic_t _shutdown_requested = 0;
 static int _systemd_activation = 0;
 
-static void _exit_handler(int sig __attribute__((unused)))
+static void _exit_handler(int sig)
 {
-	_shutdown_requested = 1;
+	_shutdown_requested = sig;
 }
 
 #define EXIT_ALREADYRUNNING 13
@@ -249,7 +249,7 @@ static int _open_socket(daemon_state s)
 		fprintf(stderr, "setting O_NONBLOCK on socket fd %d failed: %s\n", fd, strerror(errno));
 
 	fprintf(stderr, "[D] creating %s\n", s.socket_path);
-	if (!dm_strncpy(sockaddr.sun_path, s.socket_path, sizeof(sockaddr.sun_path))) {
+	if (!_dm_strncpy(sockaddr.sun_path, s.socket_path, sizeof(sockaddr.sun_path))) {
 		fprintf(stderr, "%s: daemon socket path too long.\n", s.socket_path);
 		goto error;
 	}
@@ -323,14 +323,17 @@ static void _remove_lockfile(const char *file)
 		perror("unlink failed");
 }
 
-static void _daemonise(daemon_state s)
+static void _daemonize(daemon_state s)
 {
 	int child_status;
 	int fd;
 	pid_t pid;
-	struct rlimit rlim;
 	struct timeval tval;
 	sigset_t my_sigset;
+	struct custom_fds custom_fds = {
+		/* Do not close fds preloaded by systemd! */
+		.out = (_systemd_activation) ? SD_FD_SOCKET_SERVER : -1
+	};
 
 	if ((fd = open("/dev/null", O_RDWR)) == -1) {
 		fprintf(stderr, "Unable to open /dev/null.\n");
@@ -340,6 +343,7 @@ static void _daemonise(daemon_state s)
 	sigemptyset(&my_sigset);
 	if (sigprocmask(SIG_SETMASK, &my_sigset, NULL) < 0) {
 		fprintf(stderr, "Unable to restore signals.\n");
+		(void) close(fd);
 		exit(EXIT_FAILURE);
 	}
 	signal(SIGTERM, &_exit_handler);
@@ -347,6 +351,7 @@ static void _daemonise(daemon_state s)
 	switch (pid = fork()) {
 	case -1:
 		perror("fork failed:");
+		(void) close(fd);
 		exit(EXIT_FAILURE);
 
 	case 0:		/* Child */
@@ -377,13 +382,17 @@ static void _daemonise(daemon_state s)
 
 	if (chdir("/")) {
 		perror("Cannot chdir to /");
+		(void) close(fd);
 		exit(1);
 	}
 
-	if ((dup2(fd, STDIN_FILENO) == -1) ||
-	    (dup2(fd, STDOUT_FILENO) == -1) ||
-	    (dup2(fd, STDERR_FILENO) == -1)) {
+	if (((fd != STDIN_FILENO) && (dup2(fd, STDIN_FILENO) == -1)) ||
+	    ((fd != STDOUT_FILENO) && (dup2(fd, STDOUT_FILENO) == -1)) ||
+	    ((fd != STDERR_FILENO) && (dup2(fd, STDERR_FILENO) == -1))) {
 		perror("Error setting terminal FDs to /dev/null");
+		if (fd > STDERR_FILENO)
+			(void) close(fd);
+		/* coverity[leaked_handle] fd is stdin/stdout/stderr */
 		exit(2);
 	}
 
@@ -392,22 +401,11 @@ static void _daemonise(daemon_state s)
 		exit(3);
 	}
 
-	/* Switch to sysconf(_SC_OPEN_MAX) ?? */
-	if (getrlimit(RLIMIT_NOFILE, &rlim) < 0)
-		fd = 256; /* just have to guess */
-	else
-		fd = rlim.rlim_cur;
-
-	for (--fd; fd > STDERR_FILENO; fd--) {
-#ifdef __linux__
-		/* Do not close fds preloaded by systemd! */
-		if (_systemd_activation && fd == SD_FD_SOCKET_SERVER)
-			continue;
-#endif
-		(void) close(fd);
-	}
+	daemon_close_stray_fds(s.name, 0, STDERR_FILENO, &custom_fds);
 
 	setsid();
+
+	/* coverity[leaked_handle] 'fd' is stdin/stdout/stderr */
 }
 
 response daemon_reply_simple(const char *id, ...)
@@ -454,7 +452,8 @@ static void *_client_thread(void *state)
 
 	buffer_init(&req.buffer);
 
-	while (1) {
+	/* Exit early if shutdown is requested */
+	while (!_shutdown_requested) {
 		if (!buffer_read(ts->client.socket_fd, &req.buffer))
 			goto fail;
 
@@ -567,6 +566,50 @@ static void _reap(daemon_state s, int waiting)
 	}
 }
 
+static void _shutdown_sockets(daemon_state *s)
+{
+	thread_state *ts;
+
+	/*
+	 * Close the listening socket BEFORE shutting down client sockets.
+	 * This prevents new connections during shutdown.
+	 */
+	if (s->socket_fd >= 0) {
+		INFO(s, "%s closing listening socket", s->name);
+		if (!_systemd_activation && s->socket_path && unlink(s->socket_path))
+			perror("unlink error");
+		if (close(s->socket_fd))
+			perror("socket close");
+		s->socket_fd = -1;
+	}
+
+	/*
+	 * Shutdown all client sockets to interrupt any blocking read() calls.
+	 * This allows client threads to exit quickly instead of waiting
+	 * for data that will never arrive.
+	 */
+	for (ts = s->threads->next; ts; ts = ts->next)
+		if (ts->client.socket_fd >= 0) {
+			INFO(s, "%s closing client socket fd %d", s->name, ts->client.socket_fd);
+			shutdown(ts->client.socket_fd, SHUT_RDWR);
+		}
+}
+
+static void _daemon_cleanup(daemon_state s, int failed)
+{
+	if (s.daemon_fini)
+		if (!s.daemon_fini(&s))
+			failed = 1;
+
+	INFO(&s, "%s shutting down", s.name);
+
+	closelog(); /* FIXME */
+	if (s.pidfile)
+		_remove_lockfile(s.pidfile);
+	if (failed)
+		exit(1);
+}
+
 void daemon_start(daemon_state s)
 {
 	int failed = 0;
@@ -590,7 +633,7 @@ void daemon_start(daemon_state s)
 #endif
 
 	if (!s.foreground)
-		_daemonise(s);
+		_daemonize(s);
 
 	s.log = &_log;
 	s.log->name = s.name;
@@ -643,14 +686,17 @@ void daemon_start(daemon_state s)
 	if (!s.foreground)
 		kill(getppid(), SIGTERM);
 
-	/*                                                                               	 * Use daemon_main for daemon-specific init and polling, or
+	/*
+	 * Use daemon_main for daemon-specific init and polling, or
 	 * use daemon_init for daemon-specific init and generic lib polling.
 	 */
 
 	if (s.daemon_main) {
 		if (!s.daemon_main(&s))
 			failed = 1;
-		goto out;
+		_shutdown_sockets(&s);
+		_daemon_cleanup(s, failed);
+		return;
 	}
 
 	if (s.daemon_init)
@@ -690,7 +736,7 @@ void daemon_start(daemon_state s)
 
 		/* s.idle == NULL equals no shutdown on timeout */
 		if (_is_idle(s)) {
-			DEBUGLOG(&s, "timeout occured");
+			DEBUGLOG(&s, "timeout occurred");
 			if (++timeout_count >= _get_max_timeouts(s)) {
 				INFO(&s, "Inactive for %d seconds. Exiting.", timeout_count);
 				break;
@@ -699,29 +745,12 @@ void daemon_start(daemon_state s)
 	}
 
 	if (_shutdown_requested)
-		INFO(&s, "%s shutdown requested", s.name);
+		INFO(&s, "%s shutdown requested by signal %d.", s.name, (int)_shutdown_requested);
+
+	_shutdown_sockets(&s);
 
 	INFO(&s, "%s waiting for client threads to finish", s.name);
 	_reap(s, 1);
-out:
-	/* If activated by systemd, do not unlink the socket - systemd takes care of that! */
-	if (!_systemd_activation && s.socket_fd >= 0)
-		if (s.socket_path && unlink(s.socket_path))
-			perror("unlink error");
 
-	if (s.socket_fd >= 0)
-		if (close(s.socket_fd))
-			perror("socket close");
-
-	if (s.daemon_fini)
-		if (!s.daemon_fini(&s))
-			failed = 1;
-
-	INFO(&s, "%s shutting down", s.name);
-
-	closelog(); /* FIXME */
-	if (s.pidfile)
-		_remove_lockfile(s.pidfile);
-	if (failed)
-		exit(1);
+	_daemon_cleanup(s, failed);
 }
